/* CHOPPER 
 *     Routines taken more or less directly from John Tonry's FORTH code,
 * to find and excise emission lines from a spectrum before correlating.
 *
 * ***  FOR THE EXTERNAL INTERFACE, ASSUMED TO BE FORTRAN, THE  ***
 * ***  PIXEL INDEX STARTS AT 1, NOT 0. ALL SUCH ARE CONVERTED  ***
 * ***  TO C-BASED (from 0) BEFORE PROCESSING, AND BACK TO      ***
 * ***  FORTRAN-BASED (from 1) AFTER PROCESSING AS NECESSARY.   ***
 *
 * CREATION  90-06-19  Bill Wyatt
 */

#include <stdio.h>
#include <math.h>

#include "poly.h"

#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define ABS(a)   ((a) < 0 ? -(a) : (a))

void free();

/* Some constants for the chopping operation. */
static float meanmult = 1.3;  /* interpolation buffer mean multiplier */
static float sigmult  = 5.0;  /* interp. buffer sigma multiplier */
static int   npoints  = 8;    /* number of points to sample for interp. */
static int   fitwidth = 32;   /* chop fit width, either side of line */
static int   chop_dim = 3;    /* fit a parabola to interpolate across lines */

/* Test interface routine, FORTRAN-callable, to replace the spectrum 
 * with its emission line limit so it can be plotted.
 */
fchopr_ (pix, nbins, initchoplist, inchop)
     float *pix;       /* spectrum */
     int   *nbins;     /* len of spectrum, pixels */
     int   *initchoplist;  /* initial chop list, pixel and +/- pairs */
                           /* *** PIXEL INDEX STARTS AT 1, NOT 0 *** */
     int   *inchop;     /* number in initial chop list */
{
    int    *choplist = NULL;      /* will be final list */
    int    nchop;                 /* and number of chops requested */
    float  *splbuf;
    int    i;

    float *herbiage(), snoopy();

    if((splbuf = herbiage(pix, *nbins, npoints, meanmult, sigmult)) == NULL)
      {
	  fprintf(stderr, "HERBIAGE returns NULL :");
	  perror("");
	  return(-1);
      }
    nchop = 
      findchops(pix, *nbins, fitwidth, splbuf, npoints, 
		initchoplist, *inchop, &choplist, &nchop);

    do_chops(pix, *nbins, fitwidth, choplist, nchop, chop_dim);

    printf(" Chopper: %d lines chopped%c\n", nchop, nchop == 0 ? '.' : ':');
    for(i=0; i<nchop; ++i) 
      printf("%3d. chop %4d (%8.2f) +/-%2d\n", i+1, 
	     choplist[i*2] + 1, pix[choplist[i*2]], choplist[i*2+1]);
    free((char *)choplist);
    free((char *)splbuf);
    return(0);
}

/* perform the chopping */
do_chops(pix, nbins, fitwidth, choplist, nchop, dim)
     float  *pix;
     int    nbins;
     int    fitwidth;
     int    *choplist;
     int    nchop;
{
    struct spoly *parab = NULL;
    int    section[4]; 
    int    nsect;
    int    i, j;
    int    lcompare();
    double evalfpoly();

    if(nchop <= 0 || choplist == NULL)  return(0);

    qsort((char *)choplist, nchop, sizeof(int) * 2, lcompare);

    for(i=0; i<nchop*2; i+=2) {
	fprintf(stderr, " Chop line at %d, +/-%d",
		choplist[i], choplist[i+1]);
	nsect = 1;
	section[1] = choplist[i] - choplist[i+1];
	section[0] = MAX(0, section[1] - fitwidth + 1);
	/* see if it overlaps with the next one - if so, advance to the next */
	while(i < (nchop-1)*2 && 
	      choplist[i+2] - choplist[i+3] <= choplist[i] + choplist[i+1])
	  {
	      nsect += 1;
	      i += 2;
	      fprintf(stderr, " and line at %d, +/-%d",
		      choplist[i], choplist[i+1]);
	  }
	section[2] = choplist[i] + choplist[i+1];
	section[3] = MIN(nbins-1, section[2] + fitwidth - 1);
	fprintf(stderr," : fit range = %d - %d, %d - %d\n",
		section[0], section[1], section[2], section[3]);

	if(mspfit(pix, nsect, section, dim, 0, &parab)) {
	    fprintf(stderr, 
		    " do_chops: fit error, chop = %d, +/- %d, %d-%d, %d-%d\n", 
		    choplist[i], choplist[i+1], 
		    section[0], section[1], section[2], section[3]);
	    return(-1);
	}

	/* interpolate across the chopped section */
	for(j = section[1] + 1; j < section[2]; ++j)
	  pix[j] = (float)evalfpoly(parab, (double)j); 
	
	free((char *)parab->coeff);
	free((char *)parab);
	parab = NULL;
    }

    return(nchop);
}

static int lcompare(a, b)
     char *a, *b;
{
    int i = *((int *)a);
    int j = *((int *)b);

    return(i-j);
}

/* Return is nfinal - number of lines to chop. Also returned is the
 * array containing the peak pixels and the +/- widths to chop.
 */
findchops(fpix, nbins, fitwidth, splbuf, nspl,
	  init_chop_list, ninit, chop_list, nfinal)
     float *fpix;             /* Input data array */
     int   nbins;             /* data arrray size */
     int   fitwidth;          /* limits at edge of spectrum */
     float *splbuf;           /* interpolation buffer */
     int   nspl;              /* size of interpolation buffer */
     int   *init_chop_list;   /* initial list of lines to be chopped, */
                              /*   pixel of peak, +/- width.          */
     int   ninit;             /* number of initial lines to be chopped */
     int   **chop_list;       /* RETURN inital list plus ones found here */
     int   *nfinal;           /* RETURN number of lines to chop total */
{
    char  *malloc(), *realloc();
    float snoopy();
    int   spikewidth();
    int   nbuf;
    int   i;

    if(*chop_list != NULL) free((char *)*chop_list);;

    nbuf = ninit + 25;
    if((*chop_list = (int *)malloc(nbuf * 2 * sizeof(int))) == NULL) {
	fprintf(stderr," Error in malloc for findchops: ");
	perror("");
	return(*nfinal = 0);
    }
    /* copy the initial lines into this list, and see if a width is
     * given or needs to be computed. Note that 1 is subtracted since
     * the input array is assumed to be from FORTRAN, where indices 
     * start at 1, not zero.
     */
    if(init_chop_list != NULL && ninit > 0) {
	for(i=0; i<ninit; i++)  {
	    (*chop_list)[i*2] = init_chop_list[i*2] - 1;	
	    if(((*chop_list)[i*2+1] = init_chop_list[i*2+1]) == 0)
	      (*chop_list)[i*2+1] = 
		spikewidth(fpix, nbins, init_chop_list[i], fitwidth);
	}
    }
    *nfinal = ninit;

    /* find a peak above the noise limit (set by the interpolation buffer) */
    for(i = MAX(3,fitwidth); i <= nbins-MAX(3, fitwidth); ++i) {
	if(fpix[i] > snoopy(splbuf, nspl, (float)i)) {
	    float peak = fpix[i-1] + fpix[i] + fpix[i+1];
	    float peakside;

	    /* The peak must also be greater than its +/- two neighbors. */
	    if(peak > (peakside = peak - fpix[i+1] + fpix[i-2]) &&
	       peak > peakside - fpix[i] + fpix[i-3] &&
	       peak > (peakside = peak - fpix[i-1] + fpix[i+2]) &&
	       peak > peakside - fpix[i] + fpix[i+3])
	      {
		  /* Make sure it's not within the limits of another
		   * line to be chopped.
		   */
		  int j;

		  for(j = 0; j < *nfinal; j++) {
		      if(i >= (*chop_list)[j*2] - (*chop_list)[j*2+1] &&
			 i <= (*chop_list)[j*2] - (*chop_list)[j*2+1])
			break;
		  }
		  if(j >= *nfinal) {  /* add to list */
		      if(*nfinal >= nbuf) {
			  nbuf *= 2;
			  *chop_list = 
			    (int *)realloc(*chop_list, nbuf * 2 * sizeof(int));
			  if(*chop_list == NULL) {
			      fprintf(stderr,
				      " Error in realloc for findchops: ");
			      perror("");
			      return(*nfinal = 0);
			  }
		      }
		      (*chop_list)[*nfinal * 2] = i;
		      (*chop_list)[*nfinal * 2 + 1] = 
			spikewidth(fpix, nbins, i, fitwidth);
		      *nfinal += 1;
		  }
	      }
	}
    }
    return(*nfinal);
}

/* Return is the +/- width around the center pixel to excise. */
spikewidth(fpix, nbins, center, fitwidth)
     register float *fpix;    /* input data array */
     int            nbins;    /* number of pixels total */
     register int   center;   /* Index of peak pixel of line to be excised */
     int            fitwidth; /* continuum fitting size, nominally 32 */
{
    float peakold,  peaknew;
    int   left = 0, right = 0;
    register int   i;
    
    if(center < fitwidth || center > nbins - fitwidth) return(0);
    
    /* go to the left (lower indices) to find the base of the line */
    peakold = fpix[center-1] + fpix[center] + fpix[center+1];
    for(i=center-1; i >= 1; --i, peakold = peaknew) {
	peaknew = peakold + fpix[i-1] - fpix[i+2];
	if(peaknew > peakold) {
	    left = center - i;
	    break;
	}
    }

    /* go to the right (higher indices) to find the base of the line */
    peakold = fpix[center-1] + fpix[center] + fpix[center+1];
    for(i=center+1; i <= nbins-1; ++i, peakold = peaknew) {
	peaknew = peakold + fpix[i+1] - fpix[i-2];
	if(peaknew > peakold) {
	    right = i - center;
	    break;
	}
    }
    return(MAX(left, right));
}

/* Given the spline points and a (floating pt) pixel number, 
 * return the interpolated spline value.
 * The spline buffer points are of the form (x0, y0, x1, y1, ... ),
 * and MUST be evenly spaced.
 */
float
snoopy(spl, nspl, pix)
     float *spl;
     int   nspl;
     float pix;
{
    int i;

    i = (int)(pix - spl[0]) / (int)(spl[2] - spl[0]);
    i = MIN(i, nspl - 2) * 2; 

    return((((pix - spl[i]) / (spl[i+2] - spl[i])) *
	    (spl[i+3] - spl[i+1])) + spl[i+1]);
}


/* Return the spline buffer points, where the buffer points are the
 * mean times the MEANMULT multiplier plus the rms times the SIGMULT
 * multiplier.
 * The spline buffer points are of the form (x0, y0, x1, y1, ... ).
 */
float *
herbiage(pix, nbins, nspline, meanmult, sigmult)
     float *pix;    /* input data set */
     int   nbins;   /* its size */
     int   nspline; /* Number of spline point samples to return */
     float meanmult;
     float sigmult;
{
    char  *malloc();
    float *spl;
    float mean, rms;
    int   npts = nbins / nspline;
    int   i;
    
    if((spl = (float *)malloc(2 * nspline * sizeof(float))) != NULL) {
	for(i=0; i<nspline; i++) {
	    (void) moment(pix, i * npts, (i+1) * npts - 1, 2.0, &mean, &rms);
	    spl[i * 2]       = (i * npts) + (npts / 2);
	    spl[(i * 2 + 1)] = mean * meanmult + rms * sigmult;
	}
    }
    return(spl);
}


/* Get the mean and std of a data set. The mean is computed twice; the
 * second time through, points more than FACTOR times the initial mean
 * are not used in computing the second mean.
 * RETURN is the final mean of the data set.
 */
moment(pix, left, right, factor, mean, rms)
     float *pix;     /* input data array */
     int   left;     /* index to first element to use */
     int   right;    /* index to last element to use */
     float factor;   /* Multiplier in rejection criterion of second pass */
     float *mean;    /* RETURNS the mean of the data set; */
     float *rms;     /* RETURNS (if non-NULL) the RMS of the data set */
{
    register float *fptr = &pix[left];
    register int   num   = right - left + 1;
    register int   i;

    float meaninit  = 0.0;
    float ssq       = 0.0;
    int   numfinal  = 0;

    *mean = 0.0;
    if(rms) *rms = 0.0;

    /* First pass; simple mean */
    for(i=0; i<num; ) meaninit += fptr[i++];
    meaninit = 2.0 * (ABS(meaninit) / (float)num);

    /* Second pass; some points excluded */
    for(i=0; i<num; ++i)
      if(fptr[i] <= meaninit) {
	  ++numfinal;
	  *mean += fptr[i];
	  ssq += fptr[i] * fptr[i];
      }
    if(numfinal) {
	*mean /= (float)numfinal;
	if(rms) {
	    ssq /= (float)numfinal;
	    *rms = (float)(sqrt((double)(ssq - *mean * *mean)));
	}
    }
    return(numfinal);
}

double
evalfpoly (poly, var)
     register struct spoly *poly;
     double var;
{
    register int	i	= poly->dim;
    register double	xvar	= var - poly->midpoint;
    register double	val	= 0.0;

    if(poly->scalefact != 0.0) xvar /= poly->scalefact;
    if(i < 0) return(0.0);
    while(i--)
      val = xvar * val + poly->coeff[i];

    return(val);  /* add in the constant term */
}
