/*
 * Polynomial fitting convenience routines
 */

#include <stdio.h>

#include "poly.h"

#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

/* SPFIT - Fit to a single section of an input array */
/* Return is 0 for sucess, -1 otherwise */
int
spfit(fx, nfit, offset, dim, sflag, dpoly)
     float   *fx;    /* input data array */
     int     nfit;   /* number of data points;
     int     offset; /* offset to beginning of data to fit */
     int     dim;    /* dimension of fit */
     int     sflag;  /* 1 = do data scaling, 0 = don't bother */
     struct spoly **dpoly;  /* RETURNED polynomial if success */
{
    double *x, *y, chisq;
    int    i;
    char   *malloc(), *realloc();

    if(*dpoly == NULL) {
	if((*dpoly = (struct spoly *)malloc(sizeof(struct spoly))) == NULL)
	  return(-1);
	(*dpoly)->dim   = 0;
	(*dpoly)->coeff = NULL;
    }
    if((*dpoly)->coeff == NULL) {
	if(((*dpoly)->coeff = (double *)malloc(dim * sizeof(double))) == NULL)
	  return(-1);
	(*dpoly)->dim = dim;
    }
    if((*dpoly)->dim != dim) {
	if(((*dpoly)->coeff = (double *)realloc(dim * sizeof(double))) == NULL)
	  return(-1);
	(*dpoly)->dim = dim;
    }
    (*dpoly)->midpoint = (*dpoly)->scalefact = 0.0;

    /* Compute midpoint of independent variable */
    (*dpoly)->midpoint = (double)(offset + nfit/2);
    
    /* allocate d.p. space */
    if((x = (double *)malloc(nfit * 2 * sizeof(double))) == NULL) 
      return(-1);

    y = x + nfit;

    /* Compute scaling factor (average) for independent var.
     * The mysterious formula is the average of a sequence of 
     * integers from m to n, where m = offset and n = offset+nfit-1.
     */
    if(sflag) {  
	(*dpoly)->scalefact = 
	  (nfit * (nfit + offset * 2 - 1)) / 4; 

	for(i=0; i<nfit; ++i) {
	    x[i] = ((double)(offset + i) - (*dpoly)->midpoint) / 
	      (*dpoly)->scalefact;
	    y[i] = fx[i];
	}
    } 
    else {
	for(i=0; i<nfit; ++i) {
	    x[i] = (double)(offset + i) - (*dpoly)->midpoint;
	    y[i] = (double)fx[offset + i];
	}
    }
    /* Call the fitting routine */
    polfit_(x, y, &nfit, &(*dpoly)->dim, (*dpoly)->coeff, &chisq);

    free((char *)x);

    if(chisq <= 0.0) {
	free((char *)(*dpoly)->coeff);
	free((char *)*dpoly);
	*dpoly = NULL;
    }
    return(-(chisq <= 0.0));
}

/* MSPFIT - fit to multiple sections of an input array */
/* Return is 0 for sucess, -1 otherwise */
int
mspfit(fx, nsect, sections, dim, sflag, dpoly)
     float   *fx;    /* input data array */
     int     nsect;  /* number of data sections */
     int     *sections; /* pairs of (begin,end) points to fit */
     int     dim;    /* dimension of fit */
     int     sflag;  /* 1 = do data scaling, 0 = don't bother */
     struct spoly **dpoly;  /* RETURNED polynomial if success */
{
    double *x, *y, chisq;
    int    nfit = 0;
    int    nlow, nhigh;
    int    i, j, count;
    char   *malloc(), *realloc();

    if(nsect == 0 || sections == NULL) return(-1);

    if(*dpoly == NULL) {
	if((*dpoly = (struct spoly *)malloc(sizeof(struct spoly))) == NULL)
	  return(-1);
	(*dpoly)->dim   = 0;
	(*dpoly)->coeff = NULL;
    }
    if((*dpoly)->coeff == NULL) {
	if(((*dpoly)->coeff = (double *)malloc(dim * sizeof(double))) == NULL)
	  return(-1);
	(*dpoly)->dim = dim;
    }
    if((*dpoly)->dim != dim) {
	if(((*dpoly)->coeff = (double *)realloc(dim * sizeof(double))) == NULL)
	  return(-1);
	(*dpoly)->dim = dim;
    }
    (*dpoly)->midpoint = (*dpoly)->scalefact = 0.0;

    /* Compute midpoint of independent variable */
    nlow  = sections[0];
    nhigh = sections[1];
    for(i=0; i<nsect*2; i+=2) {
	nfit += sections[i+1] - sections[i] + 1;
	nlow  = MIN(sections[i], nlow);
	nhigh = MAX(sections[i+1], nhigh);
    }
    (*dpoly)->midpoint = (double)(nhigh + nlow) / 2.0;
    
    /* allocate d.p. space */
    if((x = (double *)malloc(nfit * 2 * sizeof(double))) == NULL) 
      return(-1);

    y = x + nfit;

    /* Compute scaling factor (average) for independent var.
     * The mysterious formula is the average of a sequence of 
     * integers from m to n inclusive, where m = nlow and n = nhigh.
     */
    count = 0;
    if(sflag) {  
	(*dpoly)->scalefact = 
	  (nhigh * (nhigh + 1) - nlow * (nlow - 1)) / 4;

	for(i=0; i<nsect*2; i+=2) 
	  for(j=sections[i]; j<=sections[i+1]; ++j, ++count) {
	      x[count] = ((double)j - (*dpoly)->midpoint) / 
		(*dpoly)->scalefact;
	      y[count] = (double)fx[j];
	}
    } 
    else {
	for(i=0; i<nsect*2; i+=2) 
	  for(j=sections[i]; j<=sections[i+1]; ++j, ++count) {
	      x[count] = (double)j - (*dpoly)->midpoint;
	      y[count] = (double)fx[j];
	}
    }
    /* Call the fitting routine */
    polfit_(x, y, &nfit, &(*dpoly)->dim, (*dpoly)->coeff, &chisq);

    free((char *)x);

    if(chisq <= 0.0) {
	free((char *)(*dpoly)->coeff);
	free((char *)*dpoly);
	*dpoly = NULL;
    }
    return(-(chisq <= 0.0));
}
