# File archive/Src/arc_dispcor1.x
# November 24, 1999
# NOAO onedspec.dispcor, modified by Doug Mink

# ARC_DISPCOR -- Dispersion correct TDC archive spectra.

include	<error.h>
include	<imhdr.h>
include	<math/iminterp.h>

# Dispersion inversion parameters.
define	NIT	10	# Max interations in determining dispersion inversion
define	DX	1.d-5	# Accuracy limit in pixels for dispersion inversion

# The input spectra are corrected to a linear (or logarithmically
# linear) wavelength scale.  The dispersion function is obtained from
# the archive file.
# 
# The linear wavelength scale may be specfied by any combination of
# starting and ending wavelength, wavelength interval, and number of
# pixels.  Suitable defaults may be used from the limits of the
# dispersion solutions.
# 
# The spectra are interpolated using one of the standard image
# interpolation types and the flux may be conserved by integrating the
# interpolation function.

procedure arc_dispcor1 (out, arcdata, arctype)

pointer out			# Output file header structure
pointer	arcdata			# Archive spectrum data
int	arctype			# Archive spectrum data type

pointer	arcspec			# Buffer to hold one spectrum
int	order			# order of wavelength fit to use
bool	log			# Log-wavelength parameters?
#bool	chop			# Chop emission lines?
bool	flux			# Conserve flux?
int	npix			# Number of pixels per line in archive file
int	nw, ispec, nspec
double	w1, w2, dw
char	str[SZ_LINE]
double	clgetd()
int	clgeti()
bool	clgetb()
pointer	realdata		# Buffer for real version of input
pointer	outdata			# Output IRAF image buffer
pointer	impl2r()

include "rarc.com"
include "wpconv.com"

begin

	# Task parameters
	log = clgetb ("log")
	flux = clgetb ("flux")
	order = clgeti ("order")
	specno = clgeti ("specno")
	nspec = 1
	if (specno < 1)
	    nspec = 2
	w1 = INDEFD
	w1 = clgetd ("wl1")
	w2 = INDEFD
	w2 = clgetd ("wl2")

	# Input spectrum size
	npix = IM_LEN(out,1)

	# Output spectrum size
	nw = INDEFI
	nw = clgeti ("nwl")

	# Get the wavelength parameters and check for errors.
	call arc_wavelengths1 (npix, log, order,w1, w2, dw, nw)
	if ((dw*(w2-w1) <= 0.) || (nw < 1)) {
	    call eprintf ("Error in wavelength scale")
	    return
	    }
	IM_LEN(out,1) = nw
	IM_LEN(out,2) = 1
	if (debug ==YES) {
	    if (log)
		call printf ("ARC_DISPCOR1: %d points in log wavelength: %9.6f-%f9.6 by %8.6f\n")
	    else
		call printf ("ARC_DISPCOR1: %d points in wavelength: %9.4f-%9.4f by %7.4f\n")
		call pargi (nw)
		call pargd (w1)
		call pargd (w2)
		call pargd (dw)
	    }

	if (debug == YES) {
	    call printf ("ARC_DISPCOR1: real array allocated %d * type %d \n")
		call pargi (npix)
		call pargi (arctype)
	    }

	if (IM_PIXTYPE(out) != TY_REAL)
	    IM_PIXTYPE(out) = TY_REAL
	IM_LEN(out,1) = nw
	IM_LEN(out,2) = nspec

#  Allocate a conversion buffer

	arcspec = arcdata
	call malloc (realdata,npix,TY_REAL)

	do ispec = 1, nspec {

	# Convert the input data to floating point

	    call arc_make_real (arcspec,realdata,npix,arctype)

	# Do the dispersion correction and create the output spectrum.

	    outdata = impl2r (out,ispec)
	    call aclrr (Memr[outdata], IM_LEN(out,1))
	    if (debug == YES)
		call printf ("ARC_DISPCOR1: dispersion correction starting\n")
	    call dispcor1 (realdata, npix, outdata, w1, dw, nw, log, flux)

	#  Bump the input pointer to do the next spectrum, if there is one

	    if (ispec < nspec)
		arcspec = arcspec + npix
	    }

#  Free conversion buffer

	call mfree (realdata,TY_REAL)

#  Add header keywords.

	call imaddi (out, "DISPORD",order)
	call clgstr ("interp_type", str, SZ_LINE)
	call imastr (out, "DISPINT",str)
	if (log)
	    call imaddi (out, "DC-FLAG", 1)
	else
	    call imaddi (out, "DC-FLAG", 0)
	call imaddi (out, "DISPAXIS",dispaxis)
	call imastr (out, "DISPTYPE",disptype)
	call imastr (out, "DISPUNIT",dispunit)
	call imaddr (out, "CRPIX1", 1.)
	call imaddd (out, "CRVAL1", w1)
	call imaddd (out, "CDELT1", dw)
	call imaddd (out, "W0", w1)
	call imaddd (out, "WPC", dw)
	call imaddi (out, "NP1", 1)	# Changed from 0 to 1 11/14/1991 DJM
	call imaddi (out, "NP2", nw)
	call imaddd (out, "WL1", wblue)
	call imaddd (out, "WL2", wred)

	if (debug == YES)
	    call printf ("ARC_DISPCOR1: dispersion correction done\n")
end


# ARC_WAVELENGTHS1 -- Set output wavelength parameters for a spectrum.
# Replace INDEF parameters by suitable limits for the dispersion function.

procedure arc_wavelengths1 (npix, log, order, w1, w2, dw, nw)

int	npix		# Number of pixels per input line
bool	log		# Logarithm wavelength parameters?
int	order		# Order of wavelength solution to use (5 or 7)
double	w1, w2, dw	# Image wavelength parameters
int	nw		# Number of pixels per output line

int	i
double	arc_w2p1()

include "wpconv.com"
include "rarc.com"

begin

	if (order == 7) {
	    wblue = w7blue
	    wred = w7red
	    wdim = w7dim
	    wmid = w7mid
	    wscale = w7scale
	    do i = 1, wdim {
		wcoeff[i] = w7coeff[i]
		}
	    pdim = p7dim
	    pscale = p7scale
	    pmid = p7mid
	    do i = 1, pdim {
		pcoeff[i] = p7coeff[i]
		}
	    }
	else {
	    wblue = w5blue
	    wred = w5red
	    wdim = w5dim
	    wmid = w5mid
	    wscale = w5scale
	    do i = 1, wdim {
		wcoeff[i] = w5coeff[i]
		}
	    pdim = p5dim
	    pscale = p5scale
	    pmid = p5mid
	    do i = 1, pdim {
		pcoeff[i] = p5coeff[i]
		}
	    }
	if (w1!=INDEFD && w1>0 && w1>wblue)
	    wblue = w1
	if (w2!=INDEFD && w2>0 && w2<wred)
	    wred = w2
	if (debug == YES) {
	    call printf ("ARC_WAVELENGTHS1:  wavelength limits %f - %f\n")
	    call pargd (wblue)
	    call pargd (wred)
	    call flush (STDOUT)
	    }
	pblue = arc_w2p1 (wblue)
	pred = arc_w2p1 (wred)
	if (debug == YES) {
	    call printf ("ARC_WAVELENGTHS1:  input pixel limits %f - %f\n")
	    call pargd (pblue)
	    call pargd (pred)
	    call flush (STDOUT)
	    }
	if (log) {
	    w1 = log10 (wblue)
	    w2 = log10 (wred)
	    }
	else {
	    w1 = wblue
	    w2 = wred
	    }
	if (nw == INDEFI)
	    nw = npix
	dw = (w2 - w1) / double (nw - 1)	# changed from nw to nw-1
						# 11/14/1991 DJM
end


# Interpolation types.
define	INTERP	"|linear|poly3|poly5|spline3|"
define	LINEAR		1
define	POLY3		2
define	POLY5		3
define	SPLINE3		4

# DISPCOR -- Dispersion correct input spectrum to output spectrum.
# This procedure needs to invert the specified dispersion solutions
# and interpolate the input data, conserving flux if desired.  Image
# interpolation uses the image interpolation package and flux conservation
# integrates the interpolation function across the output pixel.  This
# procedure does some CLIO to get the interpolation function and to
# query whether to conserve flux.

procedure dispcor1 (in, npts, out, w1, dw, nw, log, flux)

pointer	in			# Input spectrum
int	npts			# Number of pixels in input spectrum
pointer	out			# Output spectrum
double	w1			# Starting wavelength
double	dw			# Wavelength per output pixel
int	nw			# Number of output pixels
bool	log			# Log scale?
bool	flux			# Conserve flux

int	i, j, i1, i2, clgwrd()
real	a, b, asieval(), asigrl()
double	x, xmin, xmax, y, dxdy, w, w1d, dwd
char	str[SZ_LINE]
#double	dx
pointer	asi
double	arc_w2p1()
pointer	outi			# pointer to current pixel of spectrum

include "wpconv.com"
include "rarc.com"

int	asitype[4]
data	asitype/II_LINEAR, II_POLY3, II_POLY5, II_SPLINE3/

begin

# Determine the range of the input spectrum over which the
# wavelength fit is good

	if (pblue < pred) {
	    xmin = pblue
	    xmax = pred
	    }
	else {
	    xmin = pred
	    xmax = pblue
	    }

	w1d = w1
	dwd = dw
	x = wblue
	y = wred
	if (log) {
	    i = ((log10 (x) - w1d) / dwd) + 1
	    j = (log10 (y) - w1d) / dwd
	    }
	else {
	    i = ((x - w1d) / dwd) + 1
	    j = (y - w1d) / dwd
	    }
	if (debug == YES) {
	    call printf ("DISPCOR1: i=%d - j=%d  w1d=%f\n")
	    call pargi (i)
	    call pargi (j)
	    call pargd (w1d)
	    call printf ("DISPCOR1: xmin=%f - xmax=%f\n")
	    call pargd (xmin)
	    call pargd (xmax)
	    }
	i1 = max (1, min (i, j) - 1)
	i2 = min (nw, max (i, j) + 1)
	dxdy = (xmax - xmin) / (y - x)

	# Initialize starting point.
	if (dwd * dxdy > 0.) {
	    y = x
	    x = 1
	    }
	else
	    x = xmax

	if (debug == YES) {
	    call printf ("DISPCOR1:  i1=%d - i2=%d > x=%f - y=%f\n")
	    call pargi (i1)
	    call pargi (i2)
	    call pargd (x)
	    call pargd (y)
	    call printf ("DISPCOR1:  dwd=%f, slope dxdy=%f\n")
	    call pargd (dwd)
	    call pargd (dxdy)
	    }

	# Determine the interpolation type, and fit the interpolation
	# function to the input spectrum.

	i = clgwrd ("interp_type", str, SZ_LINE, INTERP)
	call asiinit (asi, asitype[i])
	call asifit (asi, Memr[in], npts)
#	call printf ("DISPCOR1:  %d points, %f < x < %f\n")
#	    call pargi (npts)
#	    call pargd (xmin)
#	    call pargd (xmax)

	# If flux conserving determine edges of output pixels in input
	# spectrum and integrate using ASIGRL.  If not flux conserving
	# determine the center of each output pixel in the input spectrum
	# and evaluate the interpolation function with ASIEVAL.

	if (flux) {
	    # Determine initial left edge.
	    w = w1d + dwd * (i1 - 1.5)
	    if (log)
		w = 10. ** w
	    x = arc_w2p1 (w)
	    b = max (xmin, min (xmax, x))
	    do i = i1, i2 {
		outi = out + i - 1
	        w = w1d + dwd * (i - 0.5)
	        if (log)
		    w = 10. ** w
		x = arc_w2p1 (w)
		a = b
	        b = max (xmin, min (xmax, x))
		if (a < b)
		    Memr[outi] = asigrl (asi, a, b)
		else
		    Memr[outi] = asigrl (asi, b, a)
		j = (a + b) / 2.
#		if (debug == YES) {
#		    call printf ("DISPCOR %d: %f <- (%f %f) %d: %f\n")
#		    call pargi (i)
#		    call pargr (Memr[outi])
#		    call pargr (a)
#		    call pargr (b)
#		    call pargi (j)
#		    call pargr (in[j])
#		    }
		}
	    }
	else {
	    do i = i1, i2 {
		outi = out + i - 1
	        w = w1d + dwd * (i - 1)
		if (log)
		    w = 10. ** w
		x = arc_w2p1 (w)

		if ((x >= xmin) && (x <= xmax)) {
		    a = x
	            Memr[outi] = asieval (asi, a)
		    }
		else
	            Memr[outi] = 0.
		j = a
#		if (debug == YES) {
#		    call printf ("DISPCOR %d: %f <- (%f) %d: %f\n")
#		    call pargi (i)
#		    call pargr (Memr[outi])
#		    call pargd (x)
#		    call pargi (j)
#		    call pargr (Memr[in+j-1])
#		    }
		}
	    }

	call asifree (asi)
end


double procedure arc_w2p1 (wavelength)

double wavelength

double pixel
double wl
int i
include "wpconv.com"

begin

	pixel = 0.d0
	wl = (wavelength - pmid)
	if (pscale != 0.) wl = wl / pscale
	do i = pdim, 1, -1 {
	    pixel = pcoeff[i] + (wl * pixel)
	    }

	return (pixel)
end

# Dec  2 1991	Add DISPORD and DISPINT to header

# Feb  4 1992	Reverse xmax and xmin in dispcor if wavelength reversed
#		Drop ixmax and ixmin in dispcor--they weren't used
# Jun 17 1992	Don't add 1 pixel in w2p
# Jul  1 1992	Add arc_wavelengths1 to set first pixels correctly

# Nov 23 1999	Add 1 to diagnostic messages
# Nov 24 1999	Compare to INDEFD instead of INDEF
