include	<mach.h>
include	<imhdr.h>

define	LEN_SUM		5		# Length of statistics accumulator
define	NPTS		1		# Number of pixels
define	X		2		# Sum of pixel values
define	X2		3		# Sum of squares of pixel values
define	MIN		4		# Minimum pixel value
define	MAX		5		# Maximum pixel value


# T_IMSTATISTICS -- Compute and print the statistics of images.
#
# The statistics are formed by accumulating the information in a sum
# array.  To add additional statistics increase the size of the sum
# array and then add the necessary accumulation in the procedure
# accumulate.  Finally, add a function to calculate the statistic
# from the sum array.

procedure t_imstatistics ()

real	lower				# Lower limit of data value window
real	upper				# Upper limit of data value window

pointer	im, buf, list
char	image[SZ_FNAME]
long	v[IM_MAXDIM]
double	sum[LEN_SUM]

bool	clgetb()
int	imtgetim()
pointer	imtopenp(), imgnlr()
real	clgetr(), ist_mean(), ist_stddev()
pointer	immap()

begin
	# Open the list of input images.
	list = imtopenp ("images")

	# Get the data value limits.
	lower = clgetr ("lower")
	if (lower == INDEFR)
	    lower = -MAX_REAL
	upper = clgetr ("upper")
	if (upper == INDEFR)
	    upper = MAX_REAL

	if (clgetb ("verbose")) {
	    call printf ("##%18s  %10s  %10s  %10s  %10s  %10s\n")
		call pargstr ("IMAGE")
		call pargstr ("PIXELS")
		call pargstr ("MEAN")
		call pargstr ("STD DEV")
		call pargstr ("MINIMUM")
		call pargstr ("MAXIMUM")
	}

	# Loop through the input images.
	# Open in image, initialize the line counter and accumulation
	# array, loop through each line of the input image and 
	# accumulate the statistics.  Print the statistics and
	# close the image.

	while (imtgetim (list, image, SZ_FNAME) != EOF) {
	    im = immap (image, READ_ONLY, 0)
	    call amovkl (long(1), v, IM_MAXDIM)

	    call ist_initialize (sum)
	    while (imgnlr (im, buf, v) != EOF)
		call ist_accumulate (Memr[buf], IM_LEN(im,1), upper, lower, sum)

	    if (sum[NPTS] < 1.0) {
		sum[MIN] = INDEFR
		sum[MAX] = INDEFR
	    }

	    call printf ("%20s  %10d  %10.4g  %10.4g  %10.4g  %10.4g\n")
		call pargstr (image)
		call pargi (int (sum[NPTS]))
		call pargr (ist_mean (sum))
		call pargr (ist_stddev (sum))
		call pargd (sum[MIN])
		call pargd (sum[MAX])
	    call flush (STDOUT)
		
	    call imunmap (im)
	}

	call imtclose (list)
end


# INITIALIZE -- Initialize the sum array to zero.

procedure ist_initialize (sum)

double	sum[LEN_SUM]

begin
	sum[NPTS] = 0.
	sum[X] = 0.
	sum[X2] = 0.
	sum[MIN] = MAX_REAL
	sum[MAX] = -MAX_REAL
end


# ACCUMULATE -- Accumulate data in the sum array for the data between
# lower and upper.

procedure ist_accumulate (x, npts, upper, lower, sum)

real	x[ARB]
int	npts
real	upper
real	lower
double	sum[LEN_SUM]

int	n, i
double	xsum, xsqr
real	xmin, xmax, v

begin
	# The following idiocy is a kludge to attempt to optimize the do-loop
	# without modifying the external specifications of this procedure.

	n    = sum[NPTS]
	xsum = sum[X]
	xsqr = sum[X2]
	xmin = sum[MIN]
	xmax = sum[MAX]

	do i = 1, npts {
	    v = x[i]
	    if ((v < lower) || (v > upper))
		next

	    xsum = xsum + v
	    xsqr = xsqr + (v * v)
	    if (v > xmax)
		xmax = v
	    if (v <= xmin)
		xmin = v
	    n = n + 1
	}

	sum[NPTS] = n
	sum[X]    = xsum
	sum[X2]   = xsqr
	sum[MIN]  = xmin
	sum[MAX]  = xmax
end


# MEAN -- Compute the mean from the sum array.

real procedure ist_mean (sum)

double	sum[LEN_SUM]
real	value

begin
	if (sum[NPTS] < 1)
	    value = INDEFR
	else
	    value = sum[X] / sum[NPTS]
	
	return (value)
end


# STDDEV -- Compute the standard deviation from the sum array.

real procedure ist_stddev (sum)

double	sum[LEN_SUM]
real	value

begin
	if (sum[NPTS] < 2)
	    value = INDEFR
	else {
	    value = (sum[NPTS] * sum[X2] - sum[X] * sum[X]) /
		(sum[NPTS] * (sum[NPTS] - 1))
	    if (value <= 0.0)
		value = 0.0
	    else
		value = sqrt (value)
	}

	return (value)
end
