/* File irafpar.c
 * By Doug Mink, Center for Astrophysics
 * September 10, 2003
 * 
 * Subroutines to read, modify, and write IRAF parameter files
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h>
#include <sys/stat.h>

static char *parms;
int lparms;
int maxlparms = 0;
static char linefeed = 10;
static char temp[80];	/* value of found parameter */
char *vstart;		/* pointer to start of found parameter */
char *vend;		/* pointer to end+1 of found parameter */


/* Read IRAF parameter file */

psread (pathname)

    char *pathname;

{
    int fd;
    struct stat statbuff;

    if (stat (pathname, &statbuff)) {
	fprintf (stderr,"Cannot stat IRAF parameter file %s\n",pathname);
	exit (1);
	}
    else
	maxlparms = statbuff.st_size + 500;

    
    if ((fd = open (pathname,O_RDONLY)) < 3) {
	fprintf (stderr,"Error opening IRAF parameter file %s\n",pathname);
	exit(1);
	}

    if ((parms = malloc (maxlparms)) == NULL) {
	fprintf (stderr,"Can't malloc %d bytes for param string\n", maxlparms);
	exit(1);
	}

    if ((lparms = read (fd,parms,maxlparms)) < 1) {
	fprintf (stderr,"Error reading IRAF parameter file %s\n",pathname);
	exit(1);
	}

    close (fd);
    return;
}


/* Write IRAF parameter file */

pswrite (pathname)

    char *pathname;

{
    int fd, nw;

	if ((fd = open (pathname,O_WRONLY)) < 3) {
	    fprintf (stderr,"Error opening IRAF parameter file %s\n",pathname);
	    exit(1);
	    }

	if ((nw = write (fd,parms,lparms)) < lparms) {
	    fprintf (stderr,"Error writing IRAF parameter file %s\n",pathname);
	    exit(1);
	    }

	close (fd);

	return;
}


/* Extract character string from IRAF parameter file */

char
*psgstr (parname)
     char *parname;	/* Name of parameter to return */
{
    int slen;
    char *outstr;	/* Address of string returned */
    int psfind ();

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	slen = strlen (temp);

	if ((outstr = malloc (slen+1)) == NULL) {
	    fprintf (stderr,"Can't malloc %d bytes for string\n",slen+1);
	    exit(1);
	    }
	strncpy (outstr,temp,slen);
	outstr[slen] = 0;
	return (outstr);
	}

    return (NULL);
}


/* Implant character string into IRAF parameter file */

int
pspstr (parname, string)
     char *parname;	/* Name of parameter to set */
     char *string;	/* Value of parameter to set */
{
    int lnew, lold;
    int psfind (), atoi();
    char *pold, *pnew;

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	sprintf (temp,"%s",string);
	lnew = strlen (temp);
	lold = vend - vstart;

	/* If replacement shorter than old, add it and contract string */
	if (lnew < lold) {
	    strncpy (vstart,temp,lnew);
	    pold = vend;
	    pnew = vstart + lnew;
	    while (pold < parms + lparms) *pnew++ = *pold++;
	    }

	/* If replacement longer than old, lengthen string and add it */
	else {
	    pold = parms + lparms;
	    pnew = parms + lparms + lnew - lold;
	    while (pold > vend) *pnew-- = *pold--;
	    strncpy (vstart,temp,lnew);
	    }
	lparms = lparms + lnew - lold;
	return (1);
	}

    return (0);
}


/* Extract integer from IRAF parameter file */

int
psgeti (parname)
     char *parname;	/* Name of parameter to return */
{
    int itemp;
    int psfind (), atoi();

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	itemp = atoi (temp);
	return (itemp);
	}

    return (0);
}


/* Implant integer into IRAF parameter file */

int
psputi (parname, inum)
     char *parname;	/* Name of parameter to set */
     int inum;		/* Value of parameter to set */
{
    int lnew, lold;
    int psfind ();
    char *pold, *pnew;

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	sprintf (temp,"%d",inum);
	lnew = strlen (temp);
	lold = vend - vstart;

	/* If replacement shorter than old, add it and contract string */
	if (lnew < lold) {
	    strncpy (vstart,temp,lnew);
	    pold = vend;
	    pnew = vstart + lnew;
	    while (pold < parms + lparms) *pnew++ = *pold++;
	    }

	/* If replacement longer than old, lengthen string and add it */
	else {
	    pold = parms + lparms;
	    pnew = parms + lparms + lnew - lold;
	    while (pold > vend) *pnew-- = *pold--;
	    strncpy (vstart,temp,lnew);
	    }
	lparms = lparms + lnew - lold;
	return (1);
	}

    return (0);
}


/* Extract double precision floating point value from IRAF parameter file */

double
psgetd (parname)
     char *parname;	/* Name of parameter to return */
{
    double dtemp;
    int psfind ();

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	sscanf (temp,"%f",&dtemp);
	return (dtemp);
	}

    return (0.0);
}


/* Implant double precision floating point value into IRAF parameter file */

int
psputd (parname, dnum)
     char *parname;	/* Name of parameter to set */
     double dnum;	/* Value of parameter to set */
{
    int lnew, lold;
    int psfind ();
    char *pold, *pnew;

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	sprintf (temp,"%f",dnum);
	lnew = strlen (temp);
	lold = vend - vstart;

	/* If replacement shorter than old, add it and contract string */
	if (lnew < lold) {
	    strncpy (vstart,temp,lnew);
	    pold = vend;
	    pnew = vstart + lnew;
	    while (pold < parms + lparms) *pnew++ = *pold++;
	    }

	/* If replacement longer than old, lengthen string and add it */
	else {
	    pold = parms + lparms;
	    pnew = parms + lparms + lnew - lold;
	    while (pold > vend) *pnew-- = *pold--;
	    strncpy (vstart,temp,lnew);
	    }
	lparms = lparms + lnew - lold;
	return (1);
	}

    return (0);
}


/* Extract boolean value from IRAF parameter file */

int
psgetb (parname)
     char *parname;	/* Name of parameter to return */
{
    int itemp;
    int psfind ();

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	if (temp[0] == 'y')
	    return (1);
	else
	    return (0);
	}

    return (0);
}


/* Implant boolean value into IRAF parameter file */

int
psputb (parname, inum)
     char *parname;	/* Name of parameter to set */
     int inum;		/* Value of parameter to set */
{
    int lnew, lold;
    int psfind (), atoi();
    char *pold, *pnew;

    /* Search for parameter name in parameter string */
    if (psfind (parname) == 1) {
	if (inum == 0)
	    strcpy (temp,"no");
	else
	    strcpy (temp,"yes");
	lnew = strlen (temp);
	lold = vend - vstart;

	/* If replacement shorter than old, add it and contract string */
	if (lnew < lold) {
	    strncpy (vstart,temp,lnew);
	    pold = vend;
	    pnew = vstart + lnew;
	    while (pold < parms + lparms) *pnew++ = *pold++;
	    }

	/* If replacement longer than old, lengthen string and add it */
	else {
	    pold = parms + lparms;
	    pnew = parms + lparms + lnew - lold;
	    while (pold > vend) *pnew-- = *pold--;
	    strncpy (vstart,temp,lnew);
	    }
	lparms = lparms + lnew - lold;
	return (1);
	}

    return (0);
}


/* Search for specific parameter in IRAF parameter file */

int
psfind (parname)
     char *parname;	/* Name of parameter to return */
{
    int i, j, k, itemp, lpar;
    char *index();
    char *line,*lastchar;
    char *psearch;
    char quote = 34;
    char comma = 44;

    line = temp;
    psearch = parms;
    while (line++ < temp+80) *line = 0;

    /* Search for first character of parameter name */
    while ((line = index (psearch,parname[0]))) {

	/* Check to see if it's at the beginning of a line */
	if (line > parms) lastchar = line - 1;
	else lastchar = parms;
	if ((line == parms) || (*lastchar == linefeed)) {

	    /* see if parameter in file matches parameter name, return pointer*/
	    lpar = strlen (parname);
	    if (strncmp (parname,line,lpar) == 0) {

		/* Search for third comma */
		line = index (line,comma);
		line = line + 1;
		line = index (line,comma);
		line = line + 1;
		line = index (line,comma);
		line = line + 1;

		/* Copy until fourth comma is reached */
		j = 0;
		k = 0;
		vstart = line;
		temp[k] = 0;
		while (line[j] != comma) {
		    if (j > 80) break;
		    if (line[j] != ' ' && line[j] != quote) temp[k++] = line[j];
		    j++;
		    }
		vend = line + j;
		return (1);
		}

	    }

	/* Otherwise start searching from this point */
	psearch = line + 1;
	}

    /* Return null pointer if parmaeter name not found */
    return (0);
}


psclose ()

{
    free (parms);
    return;
}
/* Dec 30 1988	New subroutines
 *
 * Sep 10 2003	Increase size of allocated parameter string to lpar+500
 */
